︠1ec1db63-9662-4c1a-be47-399fd3dd104f︠
"""
SAGE has a large list of built-in functions and a facility for user defined functions.
In this worksheet we shall see how to access the built-in functions, how to add user defined
functions and how to see more information about them.

A list of user defined functions can be found in the SAGE REFERENCE MANUAL.
"""
︡7027aec0-2520-4208-b748-690c0ced11a7︡︡
︠aeca3390-bdd6-47f2-b050-bc5a9cdad5d9︠
#Built-in function: a demonstration.
is_prime?
︡1f7bae2e-c357-4aef-a31e-aaf803305922︡{"html": "<!--notruncate-->\n<div class=\"docstring\">\n    \n  <p><strong>File:</strong> /usr/local/sage2/local/lib/python2.6/site-packages/sage/rings/arith.py</p>\n<p><strong>Type:</strong> &lt;type &#8216;function&#8217;&gt;</p>\n<p><strong>Definition:</strong> is_prime(n)</p>\n<p><strong>Docstring:</strong></p>\n<blockquote>\n<p>Returns <tt class=\"xref docutils literal\"><span class=\"pre\">True</span></tt> if <span class=\"math\">n</span> is prime, and <tt class=\"xref docutils literal\"><span class=\"pre\">False</span></tt> otherwise.</p>\n<p>AUTHORS:</p>\n<ul class=\"simple\">\n<li>Kevin Stueve <a class=\"reference external\" href=\"mailto:kstueve&#37;&#52;&#48;uw&#46;edu\">kstueve<span>&#64;</span>uw<span>&#46;</span>edu</a> (2010-01-17):\ndelegated calculation to <tt class=\"docutils literal\"><span class=\"pre\">n.is_prime()</span></tt></li>\n</ul>\n<p>INPUT:</p>\n<ul class=\"simple\">\n<li><tt class=\"docutils literal\"><span class=\"pre\">n</span></tt> - the object for which to determine primality</li>\n</ul>\n<p>OUTPUT:</p>\n<ul class=\"simple\">\n<li><tt class=\"docutils literal\"><span class=\"pre\">bool</span></tt> - <tt class=\"xref docutils literal\"><span class=\"pre\">True</span></tt> or <tt class=\"xref docutils literal\"><span class=\"pre\">False</span></tt></li>\n</ul>\n<p>EXAMPLES:</p>\n<div class=\"highlight-python\"><div class=\"highlight\"><pre class=\"literal-block\"><span class=\"gp\">sage: </span><span class=\"n\">is_prime</span><span class=\"p\">(</span><span class=\"mf\">389</span><span class=\"p\">)</span>\n<span class=\"go\">True</span>\n<span class=\"gp\">sage: </span><span class=\"n\">is_prime</span><span class=\"p\">(</span><span class=\"mf\">2000</span><span class=\"p\">)</span>\n<span class=\"go\">False</span>\n<span class=\"gp\">sage: </span><span class=\"n\">is_prime</span><span class=\"p\">(</span><span class=\"mf\">2</span><span class=\"p\">)</span>\n<span class=\"go\">True</span>\n<span class=\"gp\">sage: </span><span class=\"n\">is_prime</span><span class=\"p\">(</span><span class=\"o\">-</span><span class=\"mf\">1</span><span class=\"p\">)</span>\n<span class=\"go\">False</span>\n<span class=\"gp\">sage: </span><span class=\"n\">factor</span><span class=\"p\">(</span><span class=\"o\">-</span><span class=\"mf\">6</span><span class=\"p\">)</span>\n<span class=\"go\">-1 * 2 * 3</span>\n<span class=\"gp\">sage: </span><span class=\"n\">is_prime</span><span class=\"p\">(</span><span class=\"mf\">1</span><span class=\"p\">)</span>\n<span class=\"go\">False</span>\n<span class=\"gp\">sage: </span><span class=\"n\">is_prime</span><span class=\"p\">(</span><span class=\"o\">-</span><span class=\"mf\">2</span><span class=\"p\">)</span>\n<span class=\"go\">False</span>\n</pre></div>\n</div>\n<p>ALGORITHM:</p>\n<p>Calculation is delegated to the <tt class=\"docutils literal\"><span class=\"pre\">n.is_prime()</span></tt> method, or in special\ncases (e.g., Python <tt class=\"docutils literal\"><span class=\"pre\">int``s)</span> <span class=\"pre\">to</span> <span class=\"pre\">``Integer(n).is_prime()</span></tt>.  If an\n<tt class=\"docutils literal\"><span class=\"pre\">n.is_prime()</span></tt> method is not available, it otherwise raises a\n<tt class=\"docutils literal\"><span class=\"pre\">TypeError</span></tt>.</p>\n</blockquote>\n\n\n</div>\n"}︡
︠52377ab2-2917-4e36-a216-d6219350c986︠
"""
Help returns four fields:
    File:   name of file implementing the object
    Type:   type of object
    Definition: Shows how the function is called
    Docstring: Displays the documentation string that was placed in the source code.
"""
︡801ee00f-c0f9-456b-a80c-c81ab102a8f2︡︡
︠86fdb01e-bdc8-4405-8fc7-5cd49d975696︠
a = factorial(90) + 1
is_prime(a)
︡2e0ca60e-5793-47f8-889a-9bd4d390c2e8︡{"stdout": "False"}︡
︠07ab11b1-4d06-4681-9a0f-92ebc9b938cc︠
#How big is 90!+1?
len(str(a))
︡1bb54c21-8ef4-4db7-a0b4-6edaae9414bc︡{"stdout": "139"}︡
︠96bb0169-6f98-4844-991b-032ea27e3f22︠
#A sample of functions, syntax, recursion and more.
def example1(num1, num2):
    """
    Returns the product of the numbers
    """
    ans = num1*num2
    return ans
︡c5228592-8b02-453a-88ea-46c223947661︡︡
︠4c57038d-5b3a-47e3-b06e-e66522508e1d︠
a = example1(87,5)
︡e7fe2a7d-2fa6-4f26-8983-edd2e20c561f︡︡
︠58c83771-9301-4ef1-8b67-8515af81f3fe︠
example1?
︡f4de6b7f-a817-4080-880a-75b18874a914︡{"html": "<!--notruncate-->\n<div class=\"docstring\"><p><strong>File:</strong> /tmp/tmpYyP2xw/___code___.py</p>\n<p><strong>Type:</strong> &lt;type 'function'&gt;</p>\n<p><strong>Definition:</strong> example1(num1, num2)</p>\n<p><strong>Docstring:</strong></p>\n<pre>\n\n\n    Returns the product of the numbers</pre></div>\n"}︡
︠c70b2a23-8aa1-4fa7-ba3a-d6ce1cd04b3a︠
print a
︡0001eb51-00b2-42b6-b76c-3e2c4f91ec2d︡{"stdout": "435"}︡
︠778c55c1-4eba-4f1d-a6bb-2b11c87e477f︠
def example2(num1):
    """
    Prints the Fibbonaci numbers smaller than num1
    """
    a, b = 0, 1 #multiple assignments a = 0, b = 1
    while b < num1:
        print b, 
        #with the "," it prints on one line; without it is printed on  separate lines    
        a, b = b, a+b
︡b9e92605-5fe7-497f-afd7-940f40c9f17b︡︡
︠e881c785-08b3-4c43-b8eb-015df241c996︠
example2(30)
︡b848c022-fc98-4d67-b39c-4002462cff45︡{"stdout": "1 1 2 3 5 8 13 21"}︡
︠661b68d9-af14-4141-a803-9fea7f267e57︠
example2(980)
︡c22342fb-ee03-4720-815a-2cf1006ff77d︡{"stdout": "1 1 2 3 5 8 13 21 34 55 89 144 233 377 610"}︡
︠2a72b8ae-1273-4c78-8cf2-96b6e48402af︠
def example3(num1, num2):
    """
    Returns the largest number. Indentation critically important!
    """
    if num1 > num2:
        ans = num1
    else:
        ans = num2
    return ans
︡ed704bc7-7d08-4655-86cc-2c7e16204921︡︡
︠207ec9ab-2a32-4db5-be52-67e6101b6db1︠
aq = example3(106, 89)
print aq
︡f164c583-9299-4834-9596-2f23032bc1f3︡{"stdout": "106"}︡
︠c9bf0709-a7be-44b5-a8f5-d265446aa84a︠
#Just another call
b1 = example3(23, example3(45,12))
print b1
︡27aa43ec-eb6a-4ef7-a853-717ceff445d4︡{"stdout": "45"}︡
︠654b16d1-2e31-4c88-b799-de1d103494c1︠
#A recursive example
def mygcd(a,b):
    """
    Returns the greatest common divisor of the integers a and b
    """
    if a*b == 0: 
        answer = a + b
    else:
        answer = mygcd(b, a%b)
    return answer
︡4fdd342f-6af5-4f04-90ad-80b66d527c83︡︡
︠6bdbcf0d-8f0e-4c7e-bafb-b4fb9612c8c1︠
fd = mygcd(879, 360)
print fd
︡ebc1d7d0-447a-4676-9112-a7d93453855a︡{"stdout": "3"}︡
︠4ad9a2ee-9bbd-407f-b739-bd0e4a9e9975︠
#In how many ways can you write n as a sum of k positive integers
def sumofIntegers(n,k):
    """
    Returns the number of different ways the positive integer n\n 
    can be expressed as a sum of k positive integers.
    """
    if (k > n) or (k <= 0):
        ans = 0
    elif (k == 1) or (k == n):
        ans = 1
    else:
        ans = sumofIntegers(n-k, k) + sumofIntegers(n-1, k-1)
    return ans
︡1bc50f09-e2d4-4040-85e8-b73b99a35348︡︡
︠fdbea0bd-4fb8-42c0-bffb-1f555e09169c︠
x = sumofIntegers(11,3)
︡752e9e30-0e8d-472a-baf4-afd1c93c1434︡︡
︠4f258016-c0c5-4fa7-a411-1b85b2d72e65︠
print x
︡52caf522-83e3-467b-9c88-a7d3647ef42d︡{"stdout": "10"}︡
︠6c0f46bc-6e55-41ed-8562-bbb41e8a1c74︠
x = sumofIntegers(11,4)
print x
︡0892a485-0152-4558-a96d-aace62b3f572︡{"stdout": "11"}︡
︠fd562247-ab9a-47b9-9d37-c69779a0c31c︠
y = sumofIntegers(23, 4)
print y
︡52421056-28fa-46c7-b46a-de7226f9c44b︡{"stdout": "94"}︡
︠e1fd14bb-b75b-479b-9c54-aac1888712f5︠
a = 2^127 + 1
is_prime(a)
︡dbc71469-afd7-4214-aebf-b2f124578e9a︡{"stdout": "False"}︡
︠522e0aee-05dc-4863-a445-8c8c07865ca9︠
def IntSize(n):
    """
    Returns the number of digits in the decimal representation of the positive integer n.
    """
    k = 0
    while n > 0:
        k = k + 1
        n = (n - n%10)/10
    return k
︡e84813b5-5db0-4400-aa42-1c51b1f4305a︡︡
︠0e152320-c5e1-43df-b219-764daca0e1fa︠
IntSize(factorial(90))
︡5e889bfa-ca95-44ff-a288-04a81b98a031︡{"stdout": "139"}︡
︠daebea19-7c9a-4984-96fa-4ecd9aa63784︠
x = factorial(90) + 1
is_prime(x)
︡8c964482-dfc6-48d4-b77b-e4d8d74058b1︡{"stdout": "False"}︡
︠d3c40335-c3d9-4fb4-b087-44c8104916b1︠
def mystery(n):
    """
    returns prime greater or equal to n
    """
    if is_prime(n):
         return n
    return mystery(n+1)
︡5480a440-14ad-4868-b0ea-bfd3b2e438b4︡︡
︠cfaf8a62-e25c-423f-a449-983c2d908d07︠
mystery(9)
︡24befa1d-66ee-40e0-bff3-1cfab595db90︡{"stdout": "11"}︡
︠c7f22c79-621e-4636-a52b-14399b89de49︠
mystery(6000)
︡cddef7bd-7662-4388-8389-d2c3c2a2dfb2︡{"stdout": "6007"}︡
︠438fb480-1601-4a09-b01f-26df88835e1c︠
#booleans
a = true; b = false; a and b; a or b;
︡b0357df3-e7fa-4af7-bbea-860557c17c06︡{"stdout": "False\nTrue"}︡
︠888bae34-faa6-47a4-8f5e-848650672cc8︠
for x in (false, true):
    for y in (false, true):
        if (x and not y):
            print "x = ",x,"y=",y
︡387784aa-7795-4fc2-9389-76c683499be8︡{"stdout": "x =  True y= False"}︡
︠b2aa3eb3-434b-4bc3-8901-08ab7587fbd6︠
#Boolean functions
def  nand(a,b):
    x = not (a and b)
    return x
︡a29f0bca-ac49-4690-bc40-53769146cef3︡︡
︠241c95c1-93a6-44f0-8e7c-d09b259f3993︠
a = true; b = false; nand(a,b)
︡fc3ca422-e6a6-4483-ab97-663d37b6a282︡{"stdout": "True"}︡
︠3b5a470f-45c7-4a40-b4d6-b546fba145dc︠
nand(a,a)
︡65a8adc0-a8e6-48c1-8842-0daf366f5f86︡{"stdout": "False"}︡
︠95ec5cb6-8db5-43c2-b34a-dc56a962070c︠
nand(b,b)
︡3445edc6-e633-49c7-a451-9e562c355f89︡{"stdout": "True"}︡
︠1c8e0aed-4fb1-42d9-9631-c6a22c2745e7︠
#the xor function
def  xor(a,b):
    x = (a == b)
    return not x

xor(a,b)
︡aa9eb6ff-0f1f-437d-bf08-79f406a61c18︡{"stdout": "True"}︡
︠3c88015d-3f0f-4c14-bfb1-a935cd962044︠
#The implies function
def implies(a,b):
    x = true
    if (a == true) and (b == false):
        x = false
    return x

implies(a,b); implies(b,a)
︡9cf0b998-ba50-4c85-84b1-6c3a67422af3︡{"stdout": "False\nTrue"}︡
